# frozen_string_literal: true

module Aqila
  module Cache
    module Methods
      extend ActiveSupport::Concern

      included do
        def self.cache_ttl=(cache_ttl)
          @cache_ttl = cache_ttl
        end

        def self.cache_ttl
          @cache_ttl || Aqila::Cache.configuration.default_ttl
        end

        def self.update_log_table
          Aqila::Cache::TableLastChange
            .find_or_initialize_by(table_name: table_name)
            .update(updated_at: Time.current)
        end

        def self.stale?(controller:)
          check_ttl

          controller.response.headers['Cache-Control'] = 'public'

          controller.stale?(last_modified: Aqila::Cache::TableLastChange.for(model: self))
        end

        def self.check_ttl
          return unless cache_ttl

          log_table = Aqila::Cache::TableLastChange.for(model: self)
          return update_log_table unless log_table

          expired = (Time.current - log_table) > cache_ttl
          update_log_table if expired
        end

        private_class_method :check_ttl
      end
    end
  end
end
